<?php
/*
Plugin Name: Twitpost
Plugin URI:  http://indeedle.com/projects/twitpost/
Description: A plugin that notifies your twitter feed when you've made a new post in your blog.
Author: Indeedle
Version: 0.0.7
Author URI: http://indeedle.com/
*/

/*  Copyright 2009  C. Finegan  (email : contact@indeedle.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

// Grab the twitter API file, this is the class we use to access twitter
require_once('twitterAPI.class.php');

// Add to the menu
add_action('admin_menu', 'twitpost_addmenu');
add_filter('wp_insert_post', 'twitpost_filterpublish', 5, 1);
register_activation_hook( __FILE__, 'twitpost_install' );
if (function_exists('register_deactivation_hook'))
	register_deactivation_hook(__FILE__, 'twitpost_uninstall');

function twitpost_addmenu()
{
	add_options_page('Twitpost', 'Twitpost', 'manage_options', __FILE__, 'twitpost_options_page');
}

// The filter
function twitpost_filterpublish($postID)
{
	// Get the post
	$post = get_post($postID);
	
	// Let's check, if the post isn't set as published then we're not going to do anything
	// Only do anything if it's both publish and we're authenticated
	
	// Secondly we need to work out, is this post being skipped?
	$status = get_post_meta($postID, 'twitpost_action', true);
	
	$options = get_option('twitpost');

	if(($post->post_status == "publish" || $status == 'post') && $options['authenticated'] == 1)
	{
		// We also need to get what the current option is
		$method = $options['method'];
		
		$twitdo = false; // Whether or not we should twitter
		
		if($method == 'man')
		{
			// To add the twitter the post needs to have a value of 'post' in the custom field
			if($status == 'post' && $status != 'added')
				$twitdo = true;
		}
		elseif($method == 'auto')
		{
			// If the post has a skip value OR has a added value, then we don't do anything
			if($status == 'skip' || $status == 'added')
				$twitdo = false;
			else
				$twitdo = true;
		}

		if($twitdo)
		{		
			// Firstly, check the validation
			$password = $options['password'];
			$username = $options['username'];
			
			$twit = new twitterAPI($username, $password);
			
			$valid = $twit->validateLogin();
			
			if($twit->validateLogin() == 1)
			{	
				// Update the custom field to posted
				update_post_meta($postID, 'twitpost_action', 'added', $status);
				
				// Add the twitter
				$message = $options['message'];
				
				// Get the variables to replace the message with
				global $current_user;
				get_currentuserinfo();
				
				$bloginfo = get_bloginfo();
				
				// Get the details	
				$user = $current_user->display_name;
				$title = $post->post_title;
				$url = $post->guid;
				$site = get_bloginfo('siteurl');
				$name = get_bloginfo('name');
				$shrinkurl = $options['shrinkurl'];
				
				/* Shrink URL Functionality */
				if($shrinkurl == 1){
					// TinyURL
					$purl = twitpost_calltinyurlAPI($url);
					
				}
				else{
					$purl = $url; // Use the normal URL
				}
				
				$twitMessage = str_replace(array('USER','TITLE','URL', 'SITE', 'NAME'), array($user, $title, $purl, $site, $name), $message);
				
				if($password != '' && $username != '')
				{
					$twit->addStatus($twitMessage);
				}
			}
			else 
			{
				// We couldn't update the status
				$options['authenticated'] = 2;
				update_option('twitpost', $options);
			}
		}
	}
	
	return $postID;
}

// Installs the plugin
function twitpost_install()
{
	// Add the options to the database
	$options = array(
		'password'		=>	'',
		'username'		=>	'',
		'authenticated'	=>	'0',
		'message'		=>	'New blog post: TITLE at URL',
		'method'		=>	'auto',
		'shrinkurl'		=>	'1'
	);
	
	// Time to check, did we actually have any old settings saved?
	// This is because changes on how we handled options have been made
	$tmp['password'] = get_option('twitpost_password');
	if(get_option('twitpost_password') != '')
		$options['password'] = $tmp['password'];
		
	$tmp['username'] = get_option('twitpost_username');
	if(get_option('twitpost_username') != '')
		$options['username'] = $tmp['username'];
		
	$tmp['message'] = get_option('twitpost_message');
	if(get_option('twitpost_message') != '')
		$options['message'] = $tmp['message'];
	
	$tmp['method'] = get_option('twitpost_method');
	if(get_option('twitpost_method') != '')
		$options['method'] = $tmp['method'];
		
	// Basically we deleted the old options from the earlier versions
	// And updated to the new one
		
	// Save them
	add_option('twitpost', $options);
	
	// To clean up the old method, get rid of the old options
	delete_option('twitpost_password');
	delete_option('twitpost_username');
	delete_option('twitpost_authenticated');
	delete_option('twitpost_message');
	delete_option('twitpost_method');
}

function twitpost_uninstall()
{
	// Let's make sure not to leave the options behind
	delete_option('twitpost');
}

// The actual options page
function twitpost_options_page()
{	
	$options = get_option('twitpost'); // We do this so we can be lazy with checking input
	// Otherwise options would be wiped
	
	if(isset($_POST['authSubmit']))
	{
		check_admin_referer('update-options');
		
		if($_POST['action'] == "update") {
			
			$options['password'] = $_POST['twitpost_password'];
			$options['username'] = $_POST['twitpost_username'];
			$options['authenticated'] = 0;
			
			update_option("twitpost", $options);
		}
	}
	elseif(isset($_POST['setSubmit']))
	{
		check_admin_referer('update-options');
		
		if($_POST['action'] == "update") {		
			$options['message'] = $_POST['twitpost_message'];
			$options['method'] = $_POST['twitpost_method'];
			$options['shrinkurl'] = $_POST['twitpost_shrinkurl'];
			
			update_option("twitpost", $options);
		}
	}
	
	// Get the clean options
	$options = get_option('twitpost');
	
	$password = stripslashes($options['password']);
	$username = stripslashes($options['username']);
	$authenticated = stripslashes($options['authenticated']);
	$message = stripslashes($options['message']);
	$shrinkurl = stripslashes($options['shrinkurl']);
	
	// Set a default message if one isn't set up
	$message = (empty($message)) ? 'USER has blogged about "TITLE" at URL' : $message;
	
	// Connect to the twitter API
	$twit = new twitterAPI($username, $password);
	
	$valid = 0;
	$err = array(); // Hold the error messages
	
	// Check, was either the password or username left blank, or has it not been previously authenticated
	if($password != '' && $username != '' && $authenticated == 0)
	{	
		// Validate login
		$valid = $twit->validateLogin();
		if($valid == 1)
		{
			// Basically by doing this we don't reauthenticate every page load
			// instead we only do it when the username/password was resubmitted
			$options['authenticated'] = 1;
			update_option('twitpost', $options);
			$authenticated = $options['authenticated'];			
		}
		elseif($valid == 2)
		{
			$err[] = 'Couldn\'t authenticate you with Twitter. Please check that your username &amp; passsword are both correct.';
		}
	}
	elseif($authenticated == 2)
	{
		// This means during the script validation failed, this can be caused by a user's password being changed
		$err[] = 'Can no longer connect to Twitter, did you change your password? Please update your login details.';
	}
	
	?><div class="wrap">
<h2>Twitpost</h2>
<p>Twitpost will add a short note to your Twitter feed when you add a new post. This can be called automatically, or it can be called manually.</p>

<h3>Authentication</h3>
<p>This is needed to update your twitter feed.</p>
<?php if(count($err) > 0){ ?><p style="padding: .5em; font-weight: bold;" class="error"><?php echo join('<br />', $err); ?></p><? } elseif($valid == 1 && $authenticated == 1){ ?><p style="padding: .5em; font-weight: bold;" class="updated fade">Great! Authenticated with twitter :)</p><? } ?>
<form action="<?php echo $_SERVER['REQUEST_URI']; ?>" method="post">
<?php wp_nonce_field('update-options'); ?>
<table class="form-table">
<tr valign="top">
<th scope="row">Twitter Username:</th>
<td><input type="text" name="twitpost_username" value="<?php echo strtolower(str_replace(array("\n", "\r", "\t", " ", "\o", "\xOB"), '', $username)); ?>" class="code" />
<span class="setting-description">Your twitter username.</span></td>
</tr><tr valign="top">
<th scope="row">Twitter Password:</th>
<td><input type="password" name="twitpost_password" value="<?php echo $password; ?>" class="code" />
<span class="setting-description">Your twitter password.</span></td>
</tr>
</table>
<input type="hidden" name="action" value="update" />
<input type="hidden" name="twitpost_authenticated" value="0" />
<input type="hidden" name="page_options" value="" />
<p class="submit">
<input type="submit" name="authSubmit" value="<?php _e('Update Login') ?>" />
</p>
</form>
<?php if($authenticated > 0){
// The login is valid, addional options to set
?>
<h3>Settings</h3>
<p>If you set the method to automatic, Twitpost will add the message to your twitter feed for every post you make, unless you specifically exclude the post.</p>
<p>With manual Twitpost will only add the message to your twitter feed if you specifically tell it to.</p>
<p>Add a custom field to your post titled <code>twitpost_action</code>. A value of <code>post</code> when method is manual means the post will be sent to twitter, while a value of <code>skip</code> when the method is automatic means the post you've written will not be sent to twitter.</p>
<p>Once the written post has been published, <code>twitpost_action</code> will be changed to <code>added</code> which means if edited in the future it will not be sent to twitter. If you want to notify twitter of it being updated, just replace <code>added</code> with <code>post</code> when editing the post, and twitter will be notified again.</p>
<form action="<?php echo $_SERVER['REQUEST_URI']; ?>" method="post">
<?php wp_nonce_field('update-options'); ?>
<table class="form-table">
<tr valign="top">
<th scope="row">Method:</th>
<td><select name="twitpost_method" class="code">
		<option class="code" value="auto"<?php if($options['method'] == 'auto') { echo " selected"; } ?>>Automatic (Upon publish)</option>
		<option class="code" value="man"<?php if($options['method'] == 'man') { echo " selected"; } ?>>Manual (Post by post)</option>
		</select><br />
<span class="setting-description"></span></td>
</tr>
<tr valign="top">
<th scope="row">Message:</th>
<td><textarea name="twitpost_message" class="large-text code"><?php echo $message; ?></textarea>
<span class="setting-description">This is the message that will be posted on twitter. <code>USER</code>, <code>TITLE</code>, <code>URL</code>, <code>SITE</code> and <code>NAME</code> will be replaced with your name, post title, post link, blog link and blog title respectivly.</span></td>
</tr>
<tr valign="top">
<th scope="row">Shrink URL:</th>
<td><select name="twitpost_shrinkurl" class="code">
<option class="code" value="0"<?php if($options['shrinkurl'] == 0) { echo " selected"; } ?>>Do not shrink</option>
<option class="code" value="1"<?php if($options['shrinkurl'] == 1) { echo " selected"; } ?>>TinyURL</option>
</select>
<span class="setting-description">Shrink the blog URL with one of these services, so the tweet doesn't appear too long. Generally the URL is the longest part of the tweet.</span></td>
</tr>
</table>
<input type="hidden" name="action" value="update" />
<input type="hidden" name="page_options" value="" />
<p class="submit">
<input type="submit" name="setSubmit" value="<?php _e('Save Settings') ?>" />
</p>
</form>
<? } ?>
</div>
<? }

// Extra functions
// Call tinyurl API
function twitpost_calltinyurlAPI($url){
	// We're going to check that CURL exists, otherwise we'll just return the url
	if(!function_exists('curl_version'))
		return $url;
	
	$encodeURL = 'http://tinyurl.com/api-create.php?url='.urlencode($url);
		
	$curl = curl_init();
    @curl_setopt ($curl, CURLOPT_URL, $encodeURL);
    @curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);

    $result = trim(curl_exec ($curl));
    @curl_close ($curl);
    
    // Check the URL returned, just to make sure there was no error
    // If it isn't a valid URL, just use the normal URL
    if(!preg_match('/^http:\/\/tinyurl.com\/([a-z]+)/', $result))
    	return $url;
    else
    	return $result;
}
?>